import signal
import socket
import sys
import time

host = "0.0.0.0"
port = 8105
buffer = 102400

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
s.bind((host, port))
s.listen(5)  # max backlog of connections


def sigint_and_hup_handler(sig, frame):
    global s
    print('SIGINT or SIGHUP received: exiting')
    try:
        s.close()
    finally:
        sys.exit(0)


signal.signal(signal.SIGINT, sigint_and_hup_handler)
signal.signal(signal.SIGHUP, sigint_and_hup_handler)

while 1:
    other = 'unknown'
    total_bytes = 0

    con, addr = s.accept()

    start = time.time()

    data = con.recv(1500)
    if data:
        header = ''
        for c in data.decode():
            if c == '\n':
                break
            else:
                header += c
        other = header
        print("Starting speed test with {}...".format(other))

        while 1:
            data = con.recv(1500)
            if not data: break
            total_bytes += len(data)
            if b'**END**' in data: break

    end = time.time()

    total_mb = total_bytes / (1024.0*1024.0)
    total_time_s =  end - start
    rate_mbps = (total_bytes*8.0 / total_time_s) / 1000000.0

    msg = '{}'.format(rate_mbps)
    con.send(msg.encode("utf-8"))

    print("Recv: from={} total={}MB rate={}Mbit/s time={}s\n"
          .format(other, total_mb, rate_mbps, total_time_s))

    con.close()
